@@ -32,6 +32,13 @@ typedef void (* ThumbnailGenFunc) (void               *type,
 				   AppearanceData     *data,
 				   GDestroyNotify     *destroy);
 
+static const gchar *symbolic_names[NUM_SYMBOLIC_COLORS] = {
+  "fg_color", "bg_color",
+  "text_color", "base_color",
+  "selected_fg_color", "selected_bg_color",
+  "tooltip_fg_color", "tooltip_bg_color"
+};
+
 static gchar *
 find_string_in_model (GtkTreeModel *model, const gchar *value, gint column)
 {
@@ -108,23 +115,26 @@ conv_to_widget_cb (GConfPropertyEditor *peditor, const GConfValue *value)
 static GConfValue *
 conv_from_widget_cb (GConfPropertyEditor *peditor, const GConfValue *value)
 {
-  GConfValue *new_value;
-  gchar *list_value = NULL;
+  GConfValue *new_value = NULL;
   GtkTreeIter iter;
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   GtkTreeView *list;
 
   list = GTK_TREE_VIEW (gconf_property_editor_get_ui_control (peditor));
-  model = gtk_tree_view_get_model (list);
-
   selection = gtk_tree_view_get_selection (list);
-  gtk_tree_selection_get_selected (selection, NULL, &iter);
-  gtk_tree_model_get (model, &iter, COL_NAME, &list_value, -1);
 
-  new_value = gconf_value_new (GCONF_VALUE_STRING);
-  gconf_value_set_string (new_value, list_value);
-  g_free (list_value);
+  if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+    gchar *list_value;
+
+    gtk_tree_model_get (model, &iter, COL_NAME, &list_value, -1);
+
+    if (list_value) {
+      new_value = gconf_value_new (GCONF_VALUE_STRING);
+      gconf_value_set_string (new_value, list_value);
+      g_free (list_value);
+    }
+  }
 
   return new_value;
 }
@@ -161,29 +171,18 @@ cursor_theme_sort_func (GtkTreeModel *model,
 static void
 update_color_buttons_from_string (const gchar *color_scheme, AppearanceData *data)
 {
-  GdkColor color_scheme_colors[8];
+  GdkColor colors[NUM_SYMBOLIC_COLORS];
   GtkWidget *widget;
+  gint i;
 
-  if (!gnome_theme_color_scheme_parse (color_scheme, color_scheme_colors))
+  if (!gnome_theme_color_scheme_parse (color_scheme, colors))
     return;
 
   /* now set all the buttons to the correct settings */
-  widget = glade_xml_get_widget (data->xml, "fg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[0]);
-  widget = glade_xml_get_widget (data->xml, "bg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[1]);
-  widget = glade_xml_get_widget (data->xml, "text_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[2]);
-  widget = glade_xml_get_widget (data->xml, "base_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[3]);
-  widget = glade_xml_get_widget (data->xml, "selected_fg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[4]);
-  widget = glade_xml_get_widget (data->xml, "selected_bg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[5]);
-  widget = glade_xml_get_widget (data->xml, "tooltip_fg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[6]);
-  widget = glade_xml_get_widget (data->xml, "tooltip_bg_colorbutton");
-  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[7]);
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i) {
+    widget = glade_xml_get_widget (data->xml, symbolic_names[i]);
+    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &colors[i]);
+  }
 }
 
 static void
@@ -193,7 +192,7 @@ update_color_buttons_from_settings (GtkSettings *settings,
   gchar *scheme, *setting;
 
   scheme = gconf_client_get_string (data->client, COLOR_SCHEME_KEY, NULL);
-  g_object_get (G_OBJECT (settings), "gtk-color-scheme", &setting, NULL);
+  g_object_get (settings, "gtk-color-scheme", &setting, NULL);
 
   if (scheme == NULL || strcmp (scheme, "") == 0)
     gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), FALSE);
@@ -218,26 +217,28 @@ check_color_schemes_enabled (GtkSettings *settings,
   gchar *theme = NULL;
   gchar *filename;
   GSList *symbolic_colors = NULL;
-  gboolean fg, bg, base, text, fg_s, bg_s, enable_colors;
+  gboolean enable_colors = FALSE;
+  gint i;
 
-  g_object_get (G_OBJECT (settings), "gtk-theme-name", &theme, NULL);
+  g_object_get (settings, "gtk-theme-name", &theme, NULL);
   filename = gtkrc_find_named (theme);
   g_free (theme);
 
   gtkrc_get_details (filename, NULL, &symbolic_colors);
   g_free (filename);
 
-  fg = (g_slist_find_custom (symbolic_colors, "fg_color", g_str_equal) != NULL);
-  bg = (g_slist_find_custom (symbolic_colors, "bg_color", g_str_equal) != NULL);
-  base = (g_slist_find_custom (symbolic_colors, "base_color", g_str_equal) != NULL);
-  text = (g_slist_find_custom (symbolic_colors, "text_color", g_str_equal) != NULL);
-  fg_s = (g_slist_find_custom (symbolic_colors, "selected_fg_color", g_str_equal) != NULL);
-  bg_s = (g_slist_find_custom (symbolic_colors, "selected_bg_color", g_str_equal) != NULL);
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i) {
+    gboolean found;
+
+    found = (g_slist_find_custom (symbolic_colors, symbolic_names[i], (GCompareFunc) strcmp) != NULL);
+    gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, symbolic_names[i]), found);
+
+    enable_colors |= found;
+  }
+
   g_slist_foreach (symbolic_colors, (GFunc) g_free, NULL);
   g_slist_free (symbolic_colors);
 
-  enable_colors = (fg && bg && base && text && fg_s && bg_s);
-
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_table"), enable_colors);
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), enable_colors);
 
@@ -250,42 +251,34 @@ check_color_schemes_enabled (GtkSettings *settings,
 static void
 color_button_clicked_cb (GtkWidget *colorbutton, AppearanceData *data)
 {
-  gchar *new_scheme;
-  GdkColor colors[NUM_SYMBOLIC_COLORS];
-  gchar *str[NUM_SYMBOLIC_COLORS + 1];
-  const gchar *widgets[NUM_SYMBOLIC_COLORS] = {
-      "fg_colorbutton", "bg_colorbutton",
-      "text_colorbutton", "base_colorbutton",
-      "selected_fg_colorbutton", "selected_bg_colorbutton",
-      "tooltip_fg_colorbutton", "tooltip_bg_colorbutton" };
-  const gchar *labels[NUM_SYMBOLIC_COLORS] = {
-      "fg_color", "bg_color",
-      "text_color", "base_color",
-      "selected_fg_color", "selected_bg_color",
-      "tooltip_fg_color", "tooltip_bg_color" };
-  gint i;
   GtkWidget *widget;
+  GdkColor color;
+  GString *scheme = g_string_new (NULL);
+  gchar *colstr;
+  gchar *old_scheme = NULL;
+  gint i;
 
   for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i) {
-    widget = glade_xml_get_widget (data->xml, widgets[i]);
-    gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[i]);
+    widget = glade_xml_get_widget (data->xml, symbolic_names[i]);
+    gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &color);
 
-    str[i] = g_strdup_printf ("%s:#%04x%04x%04x",
-             labels[i], colors[i].red, colors[i].green, colors[i].blue);
+    colstr = gdk_color_to_string (&color);
+    g_string_append_printf (scheme, "%s:%s\n", symbolic_names[i], colstr);
+    g_free (colstr);
   }
-  str[NUM_SYMBOLIC_COLORS] = NULL;
+  /* remove the last newline */
+  g_string_truncate (scheme, scheme->len - 1);
 
-  new_scheme = g_strjoinv ("\n", str);
+  /* verify that the scheme really has changed */
+  g_object_get (gtk_settings_get_default (), "gtk-color-scheme", &old_scheme, NULL);
 
-  /* Currently we assume this has only been called when one of the colours has
-   * actually changed, so we don't check the original key first */
-  gconf_client_set_string (data->client, COLOR_SCHEME_KEY, new_scheme, NULL);
+  if (!gnome_theme_color_scheme_equal (old_scheme, scheme->str)) {
+    gconf_client_set_string (data->client, COLOR_SCHEME_KEY, scheme->str, NULL);
 
-  gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), TRUE);
-
-  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i)
-    g_free (str[i]);
-  g_free (new_scheme);
+    gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), TRUE);
+  }
+  g_free (old_scheme);
+  g_string_free (scheme, TRUE);
 }
 
 static void
@@ -313,26 +306,29 @@ gtk_theme_changed (GConfPropertyEditor *peditor,
 {
   GnomeThemeInfo *theme = NULL;
   const gchar *name;
-  gchar *current_theme;
   GtkSettings *settings = gtk_settings_get_default ();
 
   if (value && (name = gconf_value_get_string (value))) {
+    gchar *current;
+
     theme = gnome_theme_info_find (name);
 
-    /* manually update GtkSettings to new gtk+ theme. */
-    g_object_get (settings, "gtk-theme-name", &current_theme, NULL);
+    /* Manually update GtkSettings to new gtk+ theme.
+     * This will eventually happen anyway, but we need the
+     * info for the color scheme updates already. */
+    g_object_get (settings, "gtk-theme-name", &current, NULL);
 
-    if (strcmp (current_theme, name) != 0)
+    if (strcmp (current, name) != 0)
       g_object_set (settings, "gtk-theme-name", name, NULL);
 
-    g_free (current_theme);
+    g_free (current);
+
+    check_color_schemes_enabled (settings, data);
+    update_color_buttons_from_settings (settings, data);
   }
 
-  check_color_schemes_enabled (settings, data);
-  update_color_buttons_from_settings (settings, data);
-
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "gtk_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_REGULAR));
+			    theme_is_writable (theme));
 }
 
 static void
@@ -348,7 +344,7 @@ window_theme_changed (GConfPropertyEditor *peditor,
     theme = gnome_theme_info_find (name);
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "window_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_REGULAR));
+			    theme_is_writable (theme));
 }
 
 static void
@@ -364,75 +360,112 @@ icon_theme_changed (GConfPropertyEditor *peditor,
     theme = gnome_theme_icon_info_find (name);
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "icon_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_ICON));
+			    theme_is_writable (theme));
 }
 
 #ifdef HAVE_XCURSOR
+static void
+cursor_size_changed_cb (int size, AppearanceData *data)
+{
+  gconf_client_set_int (data->client, CURSOR_SIZE_KEY, size, NULL);
+}
+
+static void
+cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
+{
+  GnomeThemeCursorInfo *theme;
+  gchar *name;
+
+  name = gconf_client_get_string (data->client, CURSOR_THEME_KEY, NULL);
+  if (name == NULL)
+    return;
+
+  theme = gnome_theme_cursor_info_find (name);
+  g_free (name);
+
+  if (theme) {
+    gint size;
+
+    size = g_array_index (theme->sizes, gint, (int) gtk_range_get_value (range));
+    cursor_size_changed_cb (size, data);
+  }
+}
+#endif
+
 static void
 update_cursor_size_scale (GnomeThemeCursorInfo *theme,
                           AppearanceData *data)
 {
+#ifdef HAVE_XCURSOR
   GtkWidget *cursor_size_scale;
+  GtkWidget *cursor_size_label;
+  GtkWidget *cursor_size_small_label;
+  GtkWidget *cursor_size_large_label;
+  gboolean sensitive;
+  gint size, gconf_size;
 
   cursor_size_scale = glade_xml_get_widget (data->xml, "cursor_size_scale");
+  cursor_size_label = glade_xml_get_widget (data->xml, "cursor_size_label");
+  cursor_size_small_label = glade_xml_get_widget (data->xml, "cursor_size_small_label");
+  cursor_size_large_label = glade_xml_get_widget (data->xml, "cursor_size_large_label");
 
-  if (!theme || theme->sizes->len == 1)
-  {
-    gtk_widget_set_sensitive (cursor_size_scale, FALSE);
-  }
-  else
-  {
+  sensitive = theme && theme->sizes->len > 1;
+  gtk_widget_set_sensitive (cursor_size_scale, sensitive);
+  gtk_widget_set_sensitive (cursor_size_label, sensitive);
+  gtk_widget_set_sensitive (cursor_size_small_label, sensitive);
+  gtk_widget_set_sensitive (cursor_size_large_label, sensitive);
+
+  gconf_size = gconf_client_get_int (data->client, CURSOR_SIZE_KEY, NULL);
+
+  if (sensitive) {
     GtkAdjustment *adjustment;
-    gint gconf_size, i;
-    gboolean size_found = FALSE;
+    gint i, index;
+    GtkRange *range = GTK_RANGE (cursor_size_scale);
 
-    adjustment = gtk_range_get_adjustment (GTK_RANGE (cursor_size_scale));
-    g_object_set (G_OBJECT (adjustment), "upper", (gdouble) theme->sizes->len - 1, NULL);
-    gtk_widget_set_sensitive (cursor_size_scale, TRUE);
+    adjustment = gtk_range_get_adjustment (range);
+    g_object_set (adjustment, "upper", (gdouble) theme->sizes->len - 1, NULL);
 
-    gconf_size = gconf_client_get_int (data->client, CURSOR_SIZE_KEY, NULL);
+
+    /* fallback if the gconf value is bigger than all available sizes;
+       use the largest we have */
+    index = theme->sizes->len - 1;
 
     /* set the slider to the cursor size which matches the gconf setting best  */
-    for (i = 0; i < theme->sizes->len; i++)
-    {
-      gint size;
+    for (i = 0; i < theme->sizes->len; i++) {
       size = g_array_index (theme->sizes, gint, i);
 
-      if (size == gconf_size)
-      {
-        gtk_range_set_value (GTK_RANGE (cursor_size_scale), (gdouble) i);
-        size_found = TRUE;
+      if (size == gconf_size) {
+      	index = i;
         break;
-      }
-      else if (size > gconf_size)
-      {
-        if (i == 0)
-        {
-          gtk_range_set_value (GTK_RANGE (cursor_size_scale), 0);
-          size_found = TRUE;
-          break;
-        }
-        else
-        {
+      } else if (size > gconf_size) {
+        if (i == 0) {
+          index = 0;
+        } else {
           gint diff, diff_to_last;
 
           diff = size - gconf_size;
           diff_to_last = gconf_size - g_array_index (theme->sizes, gint, i - 1);
 
-          gtk_range_set_value (GTK_RANGE (cursor_size_scale), (gdouble) (diff < diff_to_last) ? diff : diff_to_last);
-          size_found = TRUE;
-          break;
+          index = (diff < diff_to_last) ? i : i - 1;
         }
+        break;
       }
     }
 
-    /* set to the biggest size if the gconf value is bigger than
-      all available sizes */
-    if (!size_found)
-      gtk_range_set_value (GTK_RANGE (cursor_size_scale), (gdouble) g_array_index (theme->sizes, gint, theme->sizes->len - 1));
+    gtk_range_set_value (range, (gdouble) index);
+
+    size = g_array_index (theme->sizes, gint, index);
+  } else {
+    if (theme && theme->sizes->len > 0)
+      size = g_array_index (theme->sizes, gint, 0);
+    else
+      size = 18;
   }
-}
+
+  if (size != gconf_size)
+    cursor_size_changed_cb (size, data);
 #endif
+}
 
 static void
 cursor_theme_changed (GConfPropertyEditor *peditor,
@@ -446,12 +479,10 @@ cursor_theme_changed (GConfPropertyEditor *peditor,
   if (value && (name = gconf_value_get_string (value)))
     theme = gnome_theme_cursor_info_find (name);
 
-#ifdef HAVE_XCURSOR
   update_cursor_size_scale (theme, data);
-#endif
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "cursor_themes_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_CURSOR));
+			    theme_is_writable (theme));
 
 }
 
@@ -510,28 +541,11 @@ icon_theme_delete_cb (GtkWidget *button, AppearanceData *data)
   generic_theme_delete ("icon_themes_list", THEME_TYPE_ICON, data);
 }
 
-#ifdef HAVE_XCURSOR
 static void
-cursor_size_scale_value_changed_cb (GtkRange *range, AppearanceData *data)
+cursor_theme_delete_cb (GtkWidget *button, AppearanceData *data)
 {
-  GnomeThemeCursorInfo *theme;
-  gchar *name;
-
-  name = gconf_client_get_string (data->client, CURSOR_THEME_KEY, NULL);
-  if (name == NULL)
-    return;
-
-  theme = gnome_theme_cursor_info_find (name);
-  g_free (name);
-
-  if (theme) {
-    gint size;
-
-    size = g_array_index (theme->sizes, gint, (int) gtk_range_get_value (range));
-    gconf_client_set_int (data->client, CURSOR_SIZE_KEY, size, NULL);
-  }
+  generic_theme_delete ("cursor_themes_list", THEME_TYPE_CURSOR, data);
 }
-#endif
 
 static void
 add_to_treeview (const gchar *tv_name,
@@ -577,7 +591,6 @@ static void
 update_in_treeview (const gchar *tv_name,
 		    const gchar *theme_name,
 		    const gchar *theme_label,
-		    GdkPixbuf *theme_thumbnail,
 		    AppearanceData *data)
 {
   GtkTreeView *treeview;
@@ -593,7 +606,6 @@ update_in_treeview (const gchar *tv_name,
     gtk_list_store_set (model, &iter,
           COL_LABEL, theme_label,
           COL_NAME, theme_name,
-          COL_THUMBNAIL, theme_thumbnail,
           -1);
   }
 }
@@ -608,6 +620,9 @@ update_thumbnail_in_treeview (const gchar *tv_name,
   GtkListStore *model;
   GtkTreeIter iter;
 
+  if (theme_thumbnail == NULL)
+    return;
+
   treeview = GTK_TREE_VIEW (glade_xml_get_widget (data->xml, tv_name));
   model = GTK_LIST_STORE (
           gtk_tree_model_sort_get_model (
@@ -620,69 +635,6 @@ update_thumbnail_in_treeview (const gchar *tv_name,
   }
 }
 
-static void
-changed_on_disk_cb (GnomeThemeType       type,
-		    gpointer             theme,
-		    GnomeThemeChangeType change_type,
-		    GnomeThemeElement    element,
-		    AppearanceData      *data)
-{
-  if (type == GNOME_THEME_TYPE_REGULAR) {
-    GnomeThemeInfo *info = theme;
-
-    if (change_type == GNOME_THEME_CHANGE_DELETED) {
-      if (info->has_gtk)
-        remove_from_treeview ("gtk_themes_list", info->name, data);
-      if (info->has_metacity)
-        remove_from_treeview ("window_themes_list", info->name, data);
-
-    } else {
-      GdkPixbuf *thumbnail;
-
-      if (info->has_gtk) {
-      	thumbnail = generate_gtk_theme_thumbnail (info);
-
-        if (change_type == GNOME_THEME_CHANGE_CREATED)
-          add_to_treeview ("gtk_themes_list", info->name, info->name, thumbnail, data);
-        else if (change_type == GNOME_THEME_CHANGE_CHANGED)
-          update_in_treeview ("gtk_themes_list", info->name, info->name, thumbnail, data);
-
-        if (thumbnail)
-          g_object_unref (thumbnail);
-      }
-
-      if (info->has_metacity) {
-      	thumbnail = generate_metacity_theme_thumbnail (info);
-
-        if (change_type == GNOME_THEME_CHANGE_CREATED)
-          add_to_treeview ("window_themes_list", info->name, info->name, thumbnail, data);
-        else if (change_type == GNOME_THEME_CHANGE_CHANGED)
-          update_in_treeview ("window_themes_list", info->name, info->name, thumbnail, data);
-
-        if (thumbnail)
-          g_object_unref (thumbnail);
-      }
-    }
-
-  } else if (type == GNOME_THEME_TYPE_ICON) {
-    GnomeThemeIconInfo *info = theme;
-
-    if (change_type == GNOME_THEME_CHANGE_DELETED) {
-      remove_from_treeview ("icon_themes_list", info->name, data);
-    } else {
-      GdkPixbuf *thumbnail = generate_icon_theme_thumbnail (info);
-
-      if (change_type == GNOME_THEME_CHANGE_CREATED)
-        add_to_treeview ("icon_themes_list", info->name, info->readable_name, thumbnail, data);
-      else if (change_type == GNOME_THEME_CHANGE_CHANGED)
-        update_in_treeview ("icon_themes_list", info->name, info->readable_name, thumbnail, data);
-
-      if (thumbnail)
-        g_object_unref (thumbnail);
-    }
-  }
-}
-
 static void
 gtk_theme_thumbnail_cb (GdkPixbuf *pixbuf,
                         gchar *theme_name,
@@ -707,6 +659,71 @@ icon_theme_thumbnail_cb (GdkPixbuf *pixbuf,
   update_thumbnail_in_treeview ("icon_themes_list", theme_name, pixbuf, data);
 }
 
+static void
+changed_on_disk_cb (GnomeThemeCommonInfo *theme,
+		    GnomeThemeChangeType  change_type,
+		    AppearanceData       *data)
+{
+  if (theme->type == GNOME_THEME_TYPE_REGULAR) {
+    GnomeThemeInfo *info = (GnomeThemeInfo *) theme;
+
+    if (change_type == GNOME_THEME_CHANGE_DELETED) {
+      if (info->has_gtk)
+        remove_from_treeview ("gtk_themes_list", info->name, data);
+      if (info->has_metacity)
+        remove_from_treeview ("window_themes_list", info->name, data);
+
+    } else {
+      if (info->has_gtk) {
+        if (change_type == GNOME_THEME_CHANGE_CREATED)
+          add_to_treeview ("gtk_themes_list", info->name, info->name, data->gtk_theme_icon, data);
+        else if (change_type == GNOME_THEME_CHANGE_CHANGED)
+          update_in_treeview ("gtk_themes_list", info->name, info->name, data);
+
+        generate_gtk_theme_thumbnail_async (info,
+            (ThemeThumbnailFunc) gtk_theme_thumbnail_cb, data, NULL);
+      }
+
+      if (info->has_metacity) {
+        if (change_type == GNOME_THEME_CHANGE_CREATED)
+          add_to_treeview ("window_themes_list", info->name, info->name, data->window_theme_icon, data);
+        else if (change_type == GNOME_THEME_CHANGE_CHANGED)
+          update_in_treeview ("window_themes_list", info->name, info->name, data);
+
+        generate_metacity_theme_thumbnail_async (info,
+            (ThemeThumbnailFunc) metacity_theme_thumbnail_cb, data, NULL);
+      }
+    }
+
+  } else if (theme->type == GNOME_THEME_TYPE_ICON) {
+    GnomeThemeIconInfo *info = (GnomeThemeIconInfo *) theme;
+
+    if (change_type == GNOME_THEME_CHANGE_DELETED) {
+      remove_from_treeview ("icon_themes_list", info->name, data);
+    } else {
+      if (change_type == GNOME_THEME_CHANGE_CREATED)
+        add_to_treeview ("icon_themes_list", info->name, info->readable_name, data->icon_theme_icon, data);
+      else if (change_type == GNOME_THEME_CHANGE_CHANGED)
+        update_in_treeview ("icon_themes_list", info->name, info->readable_name, data);
+
+      generate_icon_theme_thumbnail_async (info,
+          (ThemeThumbnailFunc) icon_theme_thumbnail_cb, data, NULL);
+    }
+
+  } else if (theme->type == GNOME_THEME_TYPE_CURSOR) {
+    GnomeThemeCursorInfo *info = (GnomeThemeCursorInfo *) theme;
+
+    if (change_type == GNOME_THEME_CHANGE_DELETED) {
+      remove_from_treeview ("cursor_themes_list", info->name, data);
+    } else {
+      if (change_type == GNOME_THEME_CHANGE_CREATED)
+        add_to_treeview ("cursor_themes_list", info->name, info->readable_name, info->thumbnail, data);
+      else if (change_type == GNOME_THEME_CHANGE_CHANGED)
+        update_in_treeview ("cursor_themes_list", info->name, info->readable_name, data);
+    }
+  }
+}
+
 static void
 prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback callback)
 {
@@ -751,11 +768,7 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
     case THEME_TYPE_CURSOR:
       themes = gnome_theme_cursor_info_find_all ();
       thumbnail = NULL;
-#ifdef HAVE_XCURSOR
       key = CURSOR_THEME_KEY;
-#else
-      key = CURSOR_FONT_KEY;
-#endif
       generator = NULL;
       thumb_cb = NULL;
       break;
@@ -769,32 +782,18 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
 
   for (l = themes; l; l = g_list_next (l))
   {
-    const gchar *name = NULL;
-    const gchar *label = NULL;
+    GnomeThemeCommonInfo *theme = (GnomeThemeCommonInfo *) l->data;
     GtkTreeIter i;
 
-    if (type == THEME_TYPE_GTK || type == THEME_TYPE_WINDOW) {
-      name = ((GnomeThemeInfo *) l->data)->name;
-    } else if (type == THEME_TYPE_ICON) {
-      name = ((GnomeThemeIconInfo *) l->data)->name;
-      label = ((GnomeThemeIconInfo *) l->data)->readable_name;
-    } else if (type == THEME_TYPE_CURSOR) {
-      name = ((GnomeThemeCursorInfo *) l->data)->name;
-      label = ((GnomeThemeCursorInfo *) l->data)->readable_name;
-    }
-
-    if (!name)
-      continue; /* just in case... */
-
     if (type == THEME_TYPE_CURSOR) {
-      thumbnail = ((GnomeThemeCursorInfo *) l->data)->thumbnail;
+      thumbnail = ((GnomeThemeCursorInfo *) theme)->thumbnail;
     } else {
-      generator (l->data, thumb_cb, data, NULL);
+      generator (theme, thumb_cb, data, NULL);
     }
 
     gtk_list_store_insert_with_values (store, &i, 0,
-                                       COL_LABEL, label ? label : name,
-                                       COL_NAME, name,
+                                       COL_LABEL, theme->readable_name,
+                                       COL_NAME, theme->name,
                                        COL_THUMBNAIL, thumbnail,
                                        -1);
 
@@ -805,17 +804,6 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   }
   g_list_free (themes);
 
-#ifdef HAVE_XCURSOR
-  if (type == THEME_TYPE_CURSOR) {
-    GtkTreeIter i;
-    gtk_list_store_insert_with_values (store, &i, 0,
-                                       COL_LABEL, _("Default Pointer"),
-                                       COL_NAME, "default",
-                                       COL_THUMBNAIL, NULL,
-                                       -1);
-  }
-#endif
-
   sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model),
                                         COL_LABEL, GTK_SORT_ASCENDING);
@@ -828,7 +816,7 @@ prepare_list (AppearanceData *data, GtkWidget *list, ThemeType type, GCallback c
   gtk_tree_view_set_model (GTK_TREE_VIEW (list), GTK_TREE_MODEL (sort_model));
 
   renderer = gtk_cell_renderer_pixbuf_new ();
-  g_object_set (G_OBJECT (renderer), "xpad", 3, "ypad", 3, NULL);
+  g_object_set (renderer, "xpad", 3, "ypad", 3, NULL);
 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
@@ -861,8 +849,8 @@ style_init (AppearanceData *data)
 {
   GtkSettings *settings;
   GtkWidget *w;
-  GtkAdjustment *adjustment;
   gchar *label;
+  gint i;
 
   data->gtk_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/gtk-theme-thumbnailing.png", NULL);
   data->window_theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/window-theme-thumbnailing.png", NULL);
@@ -890,10 +878,7 @@ style_init (AppearanceData *data)
 
 #ifdef HAVE_XCURSOR
   w = glade_xml_get_widget (data->xml, "cursor_size_scale");
-  GTK_RANGE (w)->round_digits = 0;
-  g_signal_connect (G_OBJECT (w), "value-changed", (GCallback) cursor_size_scale_value_changed_cb, data);
-  adjustment = gtk_range_get_adjustment (GTK_RANGE (w));
-  g_object_set (G_OBJECT (adjustment), "page-size", 0.0, NULL);
+  g_signal_connect (w, "value-changed", (GCallback) cursor_size_scale_value_changed_cb, data);
 
   w = glade_xml_get_widget (data->xml, "cursor_size_small_label");
   label = g_strdup_printf ("<small><i>%s</i></small>", gtk_label_get_text (GTK_LABEL (w)));
@@ -914,20 +899,16 @@ style_init (AppearanceData *data)
 
   /* connect signals */
   /* color buttons */
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "fg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "bg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "text_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "base_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "selected_fg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "selected_bg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "tooltip_fg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "tooltip_bg_colorbutton")), "color-set", (GCallback) color_button_clicked_cb, data);
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i)
+    g_signal_connect (glade_xml_get_widget (data->xml, symbolic_names[i]), "color-set", (GCallback) color_button_clicked_cb, data);
+
   /* revert button */
-  g_signal_connect (G_OBJECT (glade_xml_get_widget (data->xml, "color_scheme_defaults_button")), "clicked", (GCallback) color_scheme_defaults_button_clicked_cb, data);
+  g_signal_connect (glade_xml_get_widget (data->xml, "color_scheme_defaults_button"), "clicked", (GCallback) color_scheme_defaults_button_clicked_cb, data);
   /* delete buttons */
   g_signal_connect (glade_xml_get_widget (data->xml, "gtk_themes_delete"), "clicked", (GCallback) gtk_theme_delete_cb, data);
   g_signal_connect (glade_xml_get_widget (data->xml, "window_themes_delete"), "clicked", (GCallback) window_theme_delete_cb, data);
   g_signal_connect (glade_xml_get_widget (data->xml, "icon_themes_delete"), "clicked", (GCallback) icon_theme_delete_cb, data);
+  g_signal_connect (glade_xml_get_widget (data->xml, "cursor_themes_delete"), "clicked", (GCallback) cursor_theme_delete_cb, data);
 
   gnome_theme_info_register_theme_change ((ThemeChangedCallback) changed_on_disk_cb, data);
 }
